.goobox-info-album {
	padding: 0;
	font-weight: bold;
}

.goobox-info-artist {
}

.goobox-info-track,
.goobox-info-time {
	color: mix (@theme_fg_color, @theme_bg_color, 0.50);
}

.goobox-info-track:backdrop,
.goobox-info-time:backdrop {
	color: mix (@theme_fg_color, @theme_bg_color, 0.75);
}

.goobox-cover-frame GtkButton:hover {
	border-image-width: 0;
	border-radius: 0;
}
